/*
 *  util.c
 *  simplepgp
 *
 *  Created by Trevor Bentley on 11/11/11.
 *
 *  Copyright 2011 Trevor Bentley
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "util.h"

uint8_t spgp_pgp_to_gcrypt_symmetric_algo(uint8_t pgpalgo) {
  switch (pgpalgo) {
  case SYM_ALGO_IDEA: return GCRY_CIPHER_IDEA;
  case SYM_ALGO_3DES: return GCRY_CIPHER_3DES;
  case SYM_ALGO_CAST5: return GCRY_CIPHER_CAST5;
  case SYM_ALGO_BLOWFISH: return GCRY_CIPHER_BLOWFISH;
  case SYM_ALGO_AES128: return GCRY_CIPHER_AES128;
  case SYM_ALGO_AES192: return GCRY_CIPHER_AES192;
  case SYM_ALGO_AES256: return GCRY_CIPHER_AES256;
  case SYM_ALGO_TWOFISH:return GCRY_CIPHER_TWOFISH;
  default: return 0xFF;
  }
}

uint8_t spgp_pgp_to_gcrypt_hash_algo(uint8_t pgpalgo) {
  switch (pgpalgo) {
  case HASH_ALGO_MD5: return GCRY_MD_MD5;
  case HASH_ALGO_SHA1: return GCRY_MD_SHA1;
  case HASH_ALGO_RIPEMD160: return GCRY_MD_RMD160;
  case HASH_ALGO_SHA256: return GCRY_MD_SHA256;
  case HASH_ALGO_SHA384: return GCRY_MD_SHA384;
  case HASH_ALGO_SHA512: return GCRY_MD_SHA512;
  case HASH_ALGO_SHA224: return GCRY_MD_SHA224;
  default: return 0xFF;
  }
}

uint8_t spgp_iv_length_for_symmetric_algo(uint8_t algo) {
  size_t ivlen = 0;
  if (gcry_cipher_algo_info(spgp_pgp_to_gcrypt_symmetric_algo(algo), 
                            GCRYCTL_GET_BLKLEN, 
                            NULL, 
                            &ivlen) != 0)
    RAISE(FORMAT_UNSUPPORTED);
  return ivlen;
}

uint8_t spgp_salt_length_for_hash_algo(uint8_t algo) {
  if (algo == HASH_ALGO_SHA1) return 8;
  else RAISE(FORMAT_UNSUPPORTED); // not implemented
  return 0;
}

